!#####################################################################
!!
!!  File  input_rcl.f90 
!!  is part of RECOLA (REcursive Computation of One Loop Amplitudes)
!!
!!  Copyright (C) 2015-2017   Stefano Actis, Ansgar Denner, 
!!                            Lars Hofer, Jean-Nicolas Lang, 
!!                            Andreas Scharf, Sandro Uccirati
!!
!!  RECOLA is licenced under the GNU GPL version 3, 
!!         see COPYING for details.
!!
!#####################################################################

  module input_rcl

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  use globals_rcl

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  implicit none

!---------------------------------------------------------------------
! Physical variables
!---------------------------------------------------------------------

! Pole Masses:
! The Z,W masses and width are computed from the on-shell masses 
! and width:
! mass_z_onshell =  91.1876d0, width_z_onshell = 2.4952d0,
! mass_w_onshell =  80.385d0,  width_w_onshell = 2.085d0,
  real(dp) :: mass_z  =  91.153480619182744d0, width_z  = 2.4942663787728243d0
  real(dp) :: mass_w  =  80.357973609877547d0, width_w  = 2.0842989982782196d0
  real(dp) :: mass_h  = 125.d0,                width_h  = 0.d0
  real(dp) :: mass_el =   0.d0
  real(dp) :: mass_mu =   0.d0,                width_mu = 0.d0
  real(dp) :: mass_ta =   0.d0,                width_ta = 0.d0
  real(dp) :: mass_u  =   0.d0
  real(dp) :: mass_d  =   0.d0
  real(dp) :: mass_c  =   0.d0,                width_c  = 0.d0
  real(dp) :: mass_s  =   0.d0
  real(dp) :: mass_t  = 173.2d0,               width_t  = 0.d0
  real(dp) :: mass_b  =   0.d0,                width_b  = 0.d0

! Regularization of collinear singularities:
! Dimensional regularization is used for collinear singularities of
! massless fermions.
! Massives fermions are treated according to the variables "light_x" 
! (x = el,mu,ta,u,d,c,s,t,b):
! - light_x = .true. 
!   Fermion "x" is light (terms proportional to the mass are 
!   discarded). Mass regularization is used for collinear divergences 
!   with mass regulator given by the pole mass of fermion "x".
! - light_x = .false.
!   Fermion "x" is heavy (full mass dependence is kept).
! If the mass of fermion "x" is zero, "light_x" is irrelevant.
  logical :: light_el = .true.
  logical :: light_mu = .true.
  logical :: light_ta = .true.
  logical :: light_u  = .true.
  logical :: light_d  = .true.
  logical :: light_c  = .true.
  logical :: light_s  = .true.
  logical :: light_t  = .false.
  logical :: light_b  = .true.

! Regularization of soft singularities:
! "reg_soft" defines the regularization of soft singularities 
! generated by massless photons and gluons:
! - reg_soft = 1: 
!   dimensional regularization is used for soft singularities 
!   ("lambda" irrelevant).
! - reg_soft = 2: 
!   mass regularization is used for soft singularities with mass 
!   regulator "lambda".
! REMARK: mass regularization for soft singularities (reg_soft = 2) 
!         is allowed only in combination with mass regularization
!         for collinear singularities of light fermions. Therefore 
!         if reg_soft = 2, no massless fermions are allowed
!         (and the code stops).
  integer :: reg_soft = 1;   real(dp) :: lambda = 100d0

! UV and IR dimensional regulators:
! "DeltaUV"  is the coefficient of the UV pole in the MSbar scheme 
!            1/ep - EulerConstant + ln(4*pi)
! "muUV"     is the UV scale
! "DeltaIR"  is the coefficient of the IR single pole
! "DeltaIR2" is the coefficient of the IR double pole
! "muIR"     is the IR scale
  real(dp) :: DeltaUV = 0d0,                 muUV = 100d0
  real(dp) :: DeltaIR = 0d0, DeltaIR2 = 0d0, muIR = 100d0

! alpha_s, renormalization scale and flavour scheme:
! "Qren" and "als" are the initialization values for the 
! renormalization scale and for alpha_s.
! "Nfren" is the number of active flavours for alpha_s renormalization. 
! "als" should correspond to the physical value of alpha_s at the 
! scale "Qren" computed in the "Nfren"-flavour scheme.
! Accepted values for "Nfren" are -1,3,4,5,6
! Nfren = -1      -> all quarks with mass less than "Qren" are active 
!                    flavours
! Nfren = 3,4,5,6 -> the 3,4,5,6 lightest quarks are the active 
!                    flavours
  real(dp) :: als   = 0.118d0
  real(dp) :: Qren  = 91.1876d0
  integer  :: Nfren = 5

! alpha, Gf and scheme for renormalization of alpha:
! - ew_reno_scheme = 1 -> gfermi scheme:
!   alpha is determined from G_F ("gf").
!   Universal corrections enhanced by large mass ratios are resummed.
!   ("al0", "alZ" are irrelevant)
! - ew_reno_scheme = 2 -> alpha0 scheme:
!   alpha is renormalized at p^2 = 0 ("al0").
!   No resummation of universal corrections
!   ("gf", "alZ" are irrelevant)
! - ew_reno_scheme = 3 -> alphaZ scheme:
!   alpha is renormalized at p^2 = mz^2 ("alZ").
!   No resummation of universal corrections
!   ("gf", "al0" are irrelevant)
  integer  :: ew_reno_scheme = 1
  real(dp) :: gf  = 1.16637d-5
  real(dp) :: al0 = 1d0/137.035999679d0
  real(dp) :: alZ = 1d0/128.936d0

! Complex mass scheme:
! - complex_mass_scheme = 1:
!   The complex mass scheme of hep-ph/0505042 is used.
! - complex_mass_scheme = 0:
!   The on-shell scheme is used, i.e. just the real part of the 
!   self-energies are taken for the computation of the counterterms. 
!   The imaginary part of the masses are kept just in the 
!   propagators (also in fermion propagator i_/(pslash-m), m is kept 
!   complex). 
  integer :: complex_mass_scheme = 1

! Self-energies of resonant particles
  logical :: resSE = .true.

! Dynamic settings
  integer :: dynamic_settings = 0

! Switch for the correction of external momenta to restore the 
! mass-shell conditions and momentum conservation
  logical :: momenta_correction = .true.

!---------------------------------------------------------------------
! Latex
!---------------------------------------------------------------------

! Generate a .tex file to draw the branches of the processes:
! 0 -> the .tex file is not generated.
! 1 -> the .tex file is generated without colour structures.
! 2 -> the .tex file is generated with colour structures.
  integer :: draw = 0

!---------------------------------------------------------------------
! Print options
!---------------------------------------------------------------------

! Print amplitudes
! writeMat = 0 -> nothing is printed.
! writeMat = 1 -> The non vanishining amplitudes A(c,h) are printed 
!                 at LO and NLO (if NLO is computed) for each coulour 
!                 structure "c" and helicity "h"
! writeMat = 2 -> In addition to writeMat=1, the NLO (if computed) 
!                 contributions to A(c,h) from bare-loop, counterterms 
!                 and rational part are also printed.
  integer :: writeMat  = 0

! Print summed squared amplitude
! writeMat2 = 0 -> nothing is printed.
! writeMat2 = 1 -> The non vanishining squared amplitudes A2 
!                  summed/averaged over colour and helicities are 
!                  printed at LO and NLO (if NLO is computed).
! writeMat2 = 2 -> In addition to writeMat2=1, the NLO (if computed) 
!                  contributions to A2 from bare-loop, counterterms 
!                  and rational part are also printed.
! writeMat2 = 3 -> In addition to writeMat2=1, also the squared 
!                  amplitudes A2h summed/averaged over colour are 
!                  printed at LO and NLO (if NLO is computed) for 
!                  each helicity "h".
  integer :: writeMat2 = 0

! Print matrix elements squared with colour correlation (if computed)
! writeCor = 0 -> nothing is printed.
! writeCor = 1 -> The non vanishining colour-correlated Born squared 
!                 amplitudes A2c(n,m) are printed for each emitter 
!                 "n" and spectator "m".
  integer :: writeCor = 0

! Print RAM needed in the present run of Recola:
! writeRAM = 0 -> nothing is printed.
! writeRAM = 1 -> RAM permanently used by process generation and process 
!                 computation is printed.
! writeRAM = 2 -> In addition to writeRAM=1, RAM temporally used by
!                 process generation is printed.
  integer :: writeRAM = 0

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  contains

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  subroutine set_pole_mass_z_rcl (m,g)

  ! This subroutine sets the pole mass ("mass_z") and width 
  ! ("width_z") of the Z boson to "m" and "g" respectively.

  real(dp), intent(in) :: m,g

  if (m.le.0d0.or.g.lt.0d0) then
    if (warnings.le.warning_limit) then
      warnings = warnings + 1
      call openOutput
      write(nx,*)
      write(nx,*) &
        'ERROR: set_pole_mass_z_rcl called with wrong arguments'
      write(nx,*)
      call toomanywarnings
    endif
    call istop (ifail,1)
  endif

  if (processes_generated) then
    if (warnings.le.warning_limit) then
      warnings = warnings + 1
      call openOutput
      write(nx,*)
      write(nx,*) 'WARNING: Processes already generated.'
      write(nx,*) '         The call of set_pole_mass_z_rcl ', &
                           'with arguments'
      write(nx,*) '          m = ',m
      write(nx,*) '          g = ',g
      write(nx,*) '         has no effects.'
      write(nx,*)
      call toomanywarnings
    endif
  endif

   mass_z = m
  width_z = g

  end subroutine set_pole_mass_z_rcl

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  subroutine set_onshell_mass_z_rcl (m,g)

  ! This subroutine sets the on-shell mass and width of the Z boson 
  ! to "m" and "g" respectively. This is done by setting the pole mass 
  ! "mass_z" to "m/sqrt(1+g^2/m^2)" and the pole width "width_z" to 
  ! "g/sqrt(1+g^2/m^2)".

  real(dp), intent(in) :: m,g

  if (m.le.0d0.or.g.lt.0d0) then
    if (warnings.le.warning_limit) then
      warnings = warnings + 1
      call openOutput
      write(nx,*)
      write(nx,*) &
        'ERROR: set_onshell_mass_z_rcl called with wrong arguments'
      write(nx,*)
      call toomanywarnings
    endif
    call istop (ifail,1)
  endif

  if (processes_generated) then
    if (warnings.le.warning_limit) then
      warnings = warnings + 1
      call openOutput
      write(nx,*)
      write(nx,*) 'WARNING: Processes already generated.'
      write(nx,*) '         The call of set_onshell_mass_z_rcl ', &
                           'with arguments'
      write(nx,*) '          m = ',m
      write(nx,*) '          g = ',g
      write(nx,*) '         has no effects.'
      write(nx,*)
      call toomanywarnings
    endif
  endif

   mass_z = m / sqrt ( 1d0 + g**2/m**2 )
  width_z = g / sqrt ( 1d0 + g**2/m**2 )

  end subroutine set_onshell_mass_z_rcl

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  subroutine set_pole_mass_w_rcl (m,g)

  ! This subroutine sets the pole mass ("mass_w") and width 
  ! ("width_w") of the W boson to "m" and "g" respectively.

  real(dp), intent(in) :: m,g

  if (m.le.0d0.or.g.lt.0d0) then
    if (warnings.le.warning_limit) then
      warnings = warnings + 1
      call openOutput
      write(nx,*)
      write(nx,*) &
        'ERROR: set_pole_mass_w_rcl called with wrong arguments'
      write(nx,*)
      call toomanywarnings
    endif
    call istop (ifail,1)
  endif

  if (processes_generated) then
    if (warnings.le.warning_limit) then
      warnings = warnings + 1
      call openOutput
      write(nx,*)
      write(nx,*) 'WARNING: Processes already generated.'
      write(nx,*) '         The call of set_pole_mass_w_rcl ', &
                           'with arguments'
      write(nx,*) '          m = ',m
      write(nx,*) '          g = ',g
      write(nx,*) '         has no effects.'
      write(nx,*)
      call toomanywarnings
    endif
  endif

   mass_w = m
  width_w = g

  end subroutine set_pole_mass_w_rcl

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  subroutine set_onshell_mass_w_rcl (m,g)

  ! This subroutine sets the on-shell mass and width of the W boson 
  ! to "m" and "g" respectively. It is done by setting the pole mass 
  ! "mass_w" to "m/sqrt(1+g^2/m^2)" and the pole width "width_w" to 
  ! "g/sqrt(1+g^2/m^2)".

  real(dp), intent(in) :: m,g

  if (m.le.0d0.or.g.lt.0d0) then
    if (warnings.le.warning_limit) then
      warnings = warnings + 1
      call openOutput
      write(nx,*)
      write(nx,*) &
        'ERROR: set_onshell_mass_w_rcl called with wrong arguments'
      write(nx,*)
      call toomanywarnings
    endif
    call istop (ifail,1)
  endif

  if (processes_generated) then
    if (warnings.le.warning_limit) then
      warnings = warnings + 1
      call openOutput
      write(nx,*)
      write(nx,*) 'WARNING: Processes already generated.'
      write(nx,*) '         The call of set_onshell_mass_w_rcl ', &
                           'with arguments'
      write(nx,*) '          m = ',m
      write(nx,*) '          g = ',g
      write(nx,*) '         has no effects.'
      write(nx,*)
      call toomanywarnings
    endif
  endif

   mass_w = m / sqrt ( 1d0 + g**2/m**2 )
  width_w = g / sqrt ( 1d0 + g**2/m**2 )

  end subroutine set_onshell_mass_w_rcl

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  subroutine set_pole_mass_h_rcl (m,g)

  ! This subroutine sets the pole mass ("mass_h") and width 
  ! ("width_h") of the Higgs boson to "m" and "g" respectively.

  real(dp), intent(in) :: m,g

  if (m.le.0d0.or.g.lt.0d0) then
    if (warnings.le.warning_limit) then
      warnings = warnings + 1
      call openOutput 
      write(nx,*)
      write(nx,*) &
        'ERROR: set_pole_mass_h_rcl called with wrong arguments'
      write(nx,*)
      call toomanywarnings
    endif
    call istop (ifail,1)
  endif

  if (processes_generated) then
    if (warnings.le.warning_limit) then
      warnings = warnings + 1
      call openOutput
      write(nx,*)
      write(nx,*) 'WARNING: Processes already generated.'
      write(nx,*) '         The call of set_pole_mass_h_rcl ', &
                           'with arguments'
      write(nx,*) '          m = ',m
      write(nx,*) '          g = ',g
      write(nx,*) '         has no effects.'
      write(nx,*)
      call toomanywarnings
    endif
  endif

   mass_h = m
  width_h = g

  end subroutine set_pole_mass_h_rcl

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  subroutine set_pole_mass_electron_rcl (m)

  ! This subroutine sets pole mass "mass_el" of the electron to "m". 

  real(dp), intent(in) :: m

  if (m.lt.0d0) then
    if (warnings.le.warning_limit) then
      warnings = warnings + 1
      call openOutput
      write(nx,*)
      write(nx,*) &
        'ERROR: set_pole_mass_electron_rcl called with negative mass'
      write(nx,*)
      call toomanywarnings
    endif
    call istop (ifail,1)
  endif

  if (processes_generated) then
    if (warnings.le.warning_limit) then
      warnings = warnings + 1
      call openOutput
      write(nx,*)
      write(nx,*) 'WARNING: Processes already generated.'
      write(nx,*) '         The call of set_pole_mass_electron_rcl ', &
                           'with argument'
      write(nx,*) '          m = ',m
      write(nx,*) '         has no effects.'
      write(nx,*)
      call toomanywarnings
    endif
  endif

  mass_el = m

  end subroutine set_pole_mass_electron_rcl

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  subroutine set_pole_mass_muon_rcl (m,g)

  ! This subroutine sets pole mass ("mass_mu") and width ("width_mu") 
  ! of the muon to "m" and "g" respectively.

  real(dp), intent(in) :: m,g

  if (m.lt.0d0.or.g.lt.0d0) then
    if (warnings.le.warning_limit) then
      warnings = warnings + 1
      call openOutput
      write(nx,*)
      write(nx,*) &
        'ERROR: set_pole_mass_muon_rcl called with wrong arguments'
      write(nx,*)
      call toomanywarnings
    endif
    call istop (ifail,1)
  endif

  if (processes_generated) then
    if (warnings.le.warning_limit) then
      warnings = warnings + 1
      call openOutput
      write(nx,*)
      write(nx,*) 'WARNING: Processes already generated.'
      write(nx,*) '         The call of set_pole_mass_muon_rcl ', &
                           'with arguments'
      write(nx,*) '          m = ',m
      write(nx,*) '          g = ',g
      write(nx,*) '         has no effects.'
      write(nx,*)
      call toomanywarnings
    endif
  endif

   mass_mu = m
  width_mu = g

  end subroutine set_pole_mass_muon_rcl

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  subroutine set_pole_mass_tau_rcl (m,g)

  ! This subroutine sets pole mass ("mass_ta") and width ("width_ta") 
  ! of the tau-lepton to "m" and "g" respectively.

  real(dp), intent(in) :: m,g

  if (m.lt.0d0.or.g.lt.0d0) then
    if (warnings.le.warning_limit) then
      warnings = warnings + 1
      call openOutput
      write(nx,*)
      write(nx,*) &
        'ERROR: set_pole_mass_tau_rcl called with wrong arguments'
      write(nx,*)
      call toomanywarnings
    endif
    call istop (ifail,1)
  endif

  if (processes_generated) then
    if (warnings.le.warning_limit) then
      warnings = warnings + 1
      call openOutput
      write(nx,*)
      write(nx,*) 'WARNING: Processes already generated.'
      write(nx,*) '         The call of set_pole_mass_tau_rcl ', &
                           'with arguments'
      write(nx,*) '          m = ',m
      write(nx,*) '          g = ',g
      write(nx,*) '         has no effects.'
      write(nx,*)
      call toomanywarnings
    endif
  endif

   mass_ta = m
  width_ta = g

  end subroutine set_pole_mass_tau_rcl

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  subroutine set_pole_mass_up_rcl (m)

  ! This subroutine sets the pole mass "mass_u" of the up-quark to 
  ! "m". 

  real(dp), intent(in) :: m

  if (m.lt.0d0) then
    if (warnings.le.warning_limit) then
      warnings = warnings + 1
      call openOutput
      write(nx,*)
      write(nx,*) &
        'ERROR: set_pole_mass_up_rcl called with negative mass'
      write(nx,*)
      call toomanywarnings
    endif
    call istop (ifail,1)
  endif

  if (processes_generated) then
    if (warnings.le.warning_limit) then
      warnings = warnings + 1
      call openOutput
      write(nx,*)
      write(nx,*) 'WARNING: Processes already generated.'
      write(nx,*) '         The call of set_pole_mass_up_rcl ', &
                           'with argument'
      write(nx,*) '          m = ',m
      write(nx,*) '         has no effects.'
      write(nx,*)
      call toomanywarnings
    endif
  endif

  mass_u = m

  end subroutine set_pole_mass_up_rcl

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  subroutine set_pole_mass_down_rcl (m)

  ! This subroutine sets the pole mass "mass_d" of the down-quark to 
  ! "m". 

  real(dp), intent(in) :: m

  if (m.lt.0d0) then
    if (warnings.le.warning_limit) then
      warnings = warnings + 1
      call openOutput
      write(nx,*)
      write(nx,*) &
        'ERROR: set_pole_mass_down_rcl called with negative mass'
      write(nx,*)
      call toomanywarnings
    endif
    call istop (ifail,1)
  endif

  if (processes_generated) then
    if (warnings.le.warning_limit) then
      warnings = warnings + 1
      call openOutput
      write(nx,*)
      write(nx,*) 'WARNING: Processes already generated.'
      write(nx,*) '         The call of set_pole_mass_down_rcl ', &
                           'with argument'
      write(nx,*) '          m = ',m
      write(nx,*) '         has no effects.'
      write(nx,*)
      call toomanywarnings
    endif
  endif

  mass_d = m

  end subroutine set_pole_mass_down_rcl

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  subroutine set_pole_mass_charm_rcl (m,g)

  ! This subroutine sets the pole mass ("mass_c") and width 
  ! ("width_c") of the charm-quark to "m" and "g" respectively.

  real(dp), intent(in) :: m,g

  if (m.lt.0d0.or.g.lt.0d0) then
    if (warnings.le.warning_limit) then
      warnings = warnings + 1
      call openOutput
      write(nx,*)
      write(nx,*) &
        'ERROR: set_pole_mass_charm_rcl called with wrong arguments'
      write(nx,*)
      call toomanywarnings
    endif
    call istop (ifail,1)
  endif

  if (processes_generated) then
    if (warnings.le.warning_limit) then
      warnings = warnings + 1
      call openOutput
      write(nx,*)
      write(nx,*) 'WARNING: Processes already generated.'
      write(nx,*) '         The call of set_pole_mass_charm_rcl ', &
                           'with arguments'
      write(nx,*) '          m = ',m
      write(nx,*) '          g = ',g
      write(nx,*) '         has no effects.'
      write(nx,*)
      call toomanywarnings
    endif
  endif

   mass_c = m
  width_c = g

  end subroutine set_pole_mass_charm_rcl

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  subroutine set_pole_mass_strange_rcl (m)

  ! This subroutine sets the pole mass "mass_s" of the strange-quark 
  ! to "m". 

  real(dp), intent(in) :: m

  if (m.lt.0d0) then
    if (warnings.le.warning_limit) then
      warnings = warnings + 1
      call openOutput
      write(nx,*)
      write(nx,*) &
        'ERROR: set_pole_mass_strange_rcl called with negative mass'
      write(nx,*)
      call toomanywarnings
    endif
    call istop (ifail,1)
  endif

  if (processes_generated) then
    if (warnings.le.warning_limit) then
      warnings = warnings + 1
      call openOutput
      write(nx,*)
      write(nx,*) 'WARNING: Processes already generated.'
      write(nx,*) '         The call of set_pole_mass_strange_rcl ', &
                           'with argument'
      write(nx,*) '          m = ',m
      write(nx,*) '         has no effects.'
      write(nx,*)
      call toomanywarnings
    endif
  endif

  mass_s = m

  end subroutine set_pole_mass_strange_rcl

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  subroutine set_pole_mass_top_rcl (m,g)

  ! This subroutine sets the pole mass ("mass_t") and width 
  ! ("width_t") of the top-quark to "m" and "g" respectively.

  real(dp), intent(in) :: m,g

  if (m.lt.0d0.or.g.lt.0d0) then
    if (warnings.le.warning_limit) then
      warnings = warnings + 1
      call openOutput
      write(nx,*)
      write(nx,*) &
        'ERROR: set_pole_mass_top_rcl called with wrong arguments'
      write(nx,*)
      call toomanywarnings
    endif
    call istop (ifail,1)
  endif

  if (processes_generated) then
    if (warnings.le.warning_limit) then
      warnings = warnings + 1
      call openOutput
      write(nx,*)
      write(nx,*) 'WARNING: Processes already generated.'
      write(nx,*) '         The call of set_pole_mass_top_rcl ', &
                           'with arguments'
      write(nx,*) '          m = ',m
      write(nx,*) '          g = ',g
      write(nx,*) '         has no effects.'
      write(nx,*)
      call toomanywarnings
    endif
  endif

   mass_t = m
  width_t = g

  end subroutine set_pole_mass_top_rcl

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  subroutine set_pole_mass_bottom_rcl (m,g)

  ! This subroutine sets the pole mass ("mass_b") and width 
  ! ("width_b") of the bottom-quark to "m" and "g" respectively.

  real(dp), intent(in) :: m,g

  if (m.lt.0d0.or.g.lt.0d0) then
    if (warnings.le.warning_limit) then
      warnings = warnings + 1
      call openOutput
      write(nx,*)
      write(nx,*) &
        'ERROR: set_pole_mass_bottom_rcl called with wrong arguments'
      write(nx,*)
      call toomanywarnings
    endif
    call istop (ifail,1)
  endif

  if (processes_generated) then
    if (warnings.le.warning_limit) then
      warnings = warnings + 1
      call openOutput
      write(nx,*)
      write(nx,*) 'WARNING: Processes already generated.'
      write(nx,*) '         The call of set_pole_mass_bottom_rcl ', &
                           'with arguments'
      write(nx,*) '          m = ',m
      write(nx,*) '          g = ',g
      write(nx,*) '         has no effects.'
      write(nx,*)
      call toomanywarnings
    endif
  endif

   mass_b = m
  width_b = g

  end subroutine set_pole_mass_bottom_rcl

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  subroutine set_light_fermions_rcl (m)

  ! This subroutine declares all massive fermions with mass smaller 
  ! than or equal to "m" as light (i.e. their collinear singularities 
  ! are regularized with their mass and terms proportional to the 
  ! mass are discarded). Massive fermions with mass larger than "m" 
  ! are heavy (i.e full mass dependence is kept). This subroutine 
  ! has no effect on massless fermions (their collinear singularities 
  ! are regularized dimensionally).

  real(dp), intent(in) :: m

  if (m.lt.0d0) then
    if (warnings.le.warning_limit) then
      warnings = warnings + 1
      call openOutput
      write(nx,*)
      write(nx,*) &
        'ERROR: set_light_fermions_rcl called with negative mass'
      write(nx,*)
      call toomanywarnings
    endif
    call istop (ifail,1)
  endif

  if (processes_generated) then
    if (warnings.le.warning_limit) then
      warnings = warnings + 1
      call openOutput
      write(nx,*)
      write(nx,*) 'WARNING: Processes already generated.'
      write(nx,*) '         The call of set_light_fermions_rcl ', &
                           'with argument'
      write(nx,*) '          m = ',m
      write(nx,*) '         has no effects.'
      write(nx,*)
      call toomanywarnings
    endif
  endif

  light_el = mass_el.le.m
  light_mu = mass_mu.le.m
  light_ta = mass_ta.le.m

  light_u = mass_u.le.m
  light_d = mass_d.le.m
  light_c = mass_c.le.m
  light_s = mass_s.le.m
  light_t = mass_t.le.m
  light_b = mass_b.le.m

  end subroutine set_light_fermions_rcl

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  subroutine set_light_electron_rcl

  ! This subroutine sets the electron as light. This means that, if 
  ! the electron is massive, its collinear singularities are 
  ! regularized with its mass and terms proportional to the mass are 
  ! discarded. If the electron is massless this subroutine has no 
  ! effect (i.e. electron collinear singularities are regularized 
  ! dimensionally).

  if (processes_generated) then
    if (warnings.le.warning_limit) then
      warnings = warnings + 1
      call openOutput
      write(nx,*)
      write(nx,*) 'WARNING: Processes already generated.'
      write(nx,*) '         The call of set_light_electron_rcl ', &
                           'has no effects.'
      write(nx,*)
      call toomanywarnings
    endif
  endif

  light_el = .true.

  end subroutine set_light_electron_rcl

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  subroutine set_light_muon_rcl

  ! This subroutine sets the muon as light. This means that, if the 
  ! muon is massive, its collinear singularities are regularized with 
  ! its mass and terms proportional to the mass are discarded. If the 
  ! muon is massless this subroutine has no effect (i.e. muon 
  ! collinear singularities are regularized dimensionally).

  if (processes_generated) then
    if (warnings.le.warning_limit) then
      warnings = warnings + 1
      call openOutput
      write(nx,*)
      write(nx,*) 'WARNING: Processes already generated.'
      write(nx,*) '         The call of set_light_muon_rcl ', &
                           'has no effects.'
      write(nx,*)
      call toomanywarnings
    endif
  endif

  light_mu = .true.

  end subroutine set_light_muon_rcl

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  subroutine set_light_tau_rcl

  ! This subroutine sets the tau-lepton as light. This means that, if 
  ! the tau-lepton is massive, its collinear singularities are 
  ! regularized with its mass and terms proportional to the mass are 
  ! discarded. If the tau-lepton is massless this subroutine has no 
  ! effect (i.e. tau-lepton collinear singularities are regularized 
  ! dimensionally).

  if (processes_generated) then
    if (warnings.le.warning_limit) then
      warnings = warnings + 1
      call openOutput
      write(nx,*)
      write(nx,*) 'WARNING: Processes already generated.'
      write(nx,*) '         The call of set_light_tau_rcl ', &
                           'has no effects.'
      write(nx,*)
      call toomanywarnings
    endif
  endif

  light_ta = .true.

  end subroutine set_light_tau_rcl

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  subroutine set_light_up_rcl

  ! This subroutine sets the up-quark as light. This means that, if 
  ! the up-quark is massive, its collinear singularities are 
  ! regularized with its mass and terms proportional to the mass are 
  ! discarded. If the up-quark is massless this subroutine has no 
  ! effect (i.e. up-quark collinear singularities are regularized 
  ! dimensionally).

  if (processes_generated) then
    if (warnings.le.warning_limit) then
      warnings = warnings + 1
      call openOutput
      write(nx,*)
      write(nx,*) 'WARNING: Processes already generated.'
      write(nx,*) '         The call of set_light_up_rcl ', &
                           'has no effects.'
      write(nx,*)
      call toomanywarnings
    endif
  endif

  light_u = .true.

  end subroutine set_light_up_rcl

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  subroutine set_light_down_rcl

  ! This subroutine sets the down quark as light. This means that, if 
  ! the down-quark is massive, its collinear singularities are 
  ! regularized with its mass and terms proportional to the mass are 
  ! discarded. If the down-quark is massless this subroutine has no 
  ! effect (i.e. down-quark collinear singularities are regularized 
  ! dimensionally).

  if (processes_generated) then
    if (warnings.le.warning_limit) then
      warnings = warnings + 1
      call openOutput
      write(nx,*)
      write(nx,*) 'WARNING: Processes already generated.'
      write(nx,*) '         The call of set_light_down_rcl ', &
                           'has no effects.'
      write(nx,*)
      call toomanywarnings
    endif
  endif

  light_d = .true.

  end subroutine set_light_down_rcl

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  subroutine set_light_charm_rcl

  ! This subroutine sets the charm quark as light. This means that, 
  ! if the charm-quark is massive, its collinear singularities are 
  ! regularized with its mass and terms proportional to the mass are 
  ! discarded. If the charm-quark is massless this subroutine has no 
  ! effect (i.e. charm-quark collinear singularities are regularized 
  ! dimensionally).

  if (processes_generated) then
    if (warnings.le.warning_limit) then
      warnings = warnings + 1
      call openOutput
      write(nx,*)
      write(nx,*) 'WARNING: Processes already generated.'
      write(nx,*) '         The call of set_light_charm_rcl ', &
                           'has no effects.'
      write(nx,*)
      call toomanywarnings
    endif
  endif

  light_c = .true.

  end subroutine set_light_charm_rcl

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  subroutine set_light_strange_rcl

  ! This subroutine sets the strange quark as light. This means that, 
  ! if the strange-quark is massive, its collinear singularities are 
  ! regularized with its mass and terms proportional to the mass are 
  ! discarded. If the strange-quark is massless this subroutine has 
  ! no effect (i.e. strange-quark collinear singularities are 
  ! regularized dimensionally).

  if (processes_generated) then
    if (warnings.le.warning_limit) then
      warnings = warnings + 1
      call openOutput
      write(nx,*)
      write(nx,*) 'WARNING: Processes already generated.'
      write(nx,*) '         The call of set_light_strange_rcl ', &
                           'has no effects.'
      write(nx,*)
      call toomanywarnings
    endif
  endif

  light_s = .true.

  end subroutine set_light_strange_rcl

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  subroutine set_light_top_rcl

  ! This subroutine sets the top quark as light. This means that, if 
  ! the top-quark is massive, its collinear singularities are 
  ! regularized with its mass and terms proportional to the mass are 
  ! discarded. If the top-quark is massless this subroutine has no 
  ! effect (i.e. top-quark collinear singularities are regularized 
  ! dimensionally).

  if (processes_generated) then
    if (warnings.le.warning_limit) then
      warnings = warnings + 1
      call openOutput
      write(nx,*)
      write(nx,*) 'WARNING: Processes already generated.'
      write(nx,*) '         The call of set_light_top_rcl ', &
                           'has no effects.'
      write(nx,*)
      call toomanywarnings
    endif
  endif

  light_t = .true.

  end subroutine set_light_top_rcl

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  subroutine set_light_bottom_rcl

  ! This subroutine sets the bottom quark as light. This means that, 
  ! if the bottom-quark is massive, its collinear singularities are 
  ! regularized with its mass and terms proportional to the mass are 
  ! discarded. If the bottom-quark is massless this subroutine has no 
  ! effect (i.e. bottom-quark collinear singularities are regularized 
  ! dimensionally).

  if (processes_generated) then
    if (warnings.le.warning_limit) then
      warnings = warnings + 1
      call openOutput
      write(nx,*)
      write(nx,*) 'WARNING: Processes already generated.'
      write(nx,*) '         The call of set_light_bottom_rcl ', &
                           'has no effects.'
      write(nx,*)
      call toomanywarnings
    endif
  endif

  light_b = .true.

  end subroutine set_light_bottom_rcl

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  subroutine unset_light_electron_rcl

  ! This subroutine sets the electron as "not light". This means 
  ! that, if the electron is massive, it is considered as heavy (i.e 
  ! full mass dependence is kept). If the electron is massless this 
  ! subroutine has no effect (i.e. electron collinear singularities 
  ! are regularized dimensionally).

  if (processes_generated) then
    if (warnings.le.warning_limit) then
      warnings = warnings + 1
      call openOutput
      write(nx,*)
      write(nx,*) 'WARNING: Processes already generated.'
      write(nx,*) '         The call of unset_light_electron_rcl ', &
                           'has no effects.'
      write(nx,*)
      call toomanywarnings
    endif
  endif

  light_el = .false.

  end subroutine unset_light_electron_rcl

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  subroutine unset_light_muon_rcl

  ! This subroutine sets the muon as "not light". This means that, if 
  ! the muon is massive, it is considered as heavy (i.e full mass 
  ! dependence is kept). If the muon is massless this subroutine has 
  ! no effect (i.e. muon collinear singularities are regularized 
  ! dimensionally).

  if (processes_generated) then
    if (warnings.le.warning_limit) then
      warnings = warnings + 1
      call openOutput
      write(nx,*)
      write(nx,*) 'WARNING: Processes already generated.'
      write(nx,*) '         The call of unset_light_muon_rcl ', &
                           'has no effects.'
      write(nx,*)
      call toomanywarnings
    endif
  endif

  light_mu = .false.

  end subroutine unset_light_muon_rcl

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  subroutine unset_light_tau_rcl

  ! This subroutine sets the tau-lepton as "not light". This means 
  ! that, if the tau-lepton is massive, it is considered as heavy (i.e 
  ! full mass dependence is kept). If the tau-lepton is massless this 
  ! subroutine has no effect (i.e. tau-lepton collinear singularities 
  ! are regularized dimensionally).

  if (processes_generated) then
    if (warnings.le.warning_limit) then
      warnings = warnings + 1
      call openOutput
      write(nx,*)
      write(nx,*) 'WARNING: Processes already generated.'
      write(nx,*) '         The call of unset_light_tau_rcl ', &
                           'has no effects.'
      write(nx,*)
      call toomanywarnings
    endif
  endif

  light_ta = .false.

  end subroutine unset_light_tau_rcl

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  subroutine unset_light_up_rcl

  ! This subroutine sets the up quark as "not light". This means 
  ! that, if the up-quark is massive, it is considered as heavy (i.e 
  ! full mass dependence is kept). If the up-quark is massless this 
  ! subroutine has no effect (i.e. up-quark collinear singularities 
  ! are regularized dimensionally).

  if (processes_generated) then
    if (warnings.le.warning_limit) then
      warnings = warnings + 1
      call openOutput
      write(nx,*)
      write(nx,*) 'WARNING: Processes already generated.'
      write(nx,*) '         The call of unset_light_up_rcl ', &
                           'has no effects.'
      write(nx,*)
      call toomanywarnings
    endif
  endif

  light_u = .false.

  end subroutine unset_light_up_rcl

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  subroutine unset_light_down_rcl

  ! This subroutine sets the down quark as "not light". This means 
  ! that, if the down-quark is massive, it is considered as heavy 
  ! (i.e full mass dependence is kept). If the down-quark is massless 
  ! this subroutine has no effect (i.e. down-quark collinear 
  ! singularities are regularized dimensionally).

  if (processes_generated) then
    if (warnings.le.warning_limit) then
      warnings = warnings + 1
      call openOutput
      write(nx,*)
      write(nx,*) 'WARNING: Processes already generated.'
      write(nx,*) '         The call of unset_light_down_rcl ', &
                           'has no effects.'
      write(nx,*)
      call toomanywarnings
    endif
  endif

  light_d = .false.

  end subroutine unset_light_down_rcl

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  subroutine unset_light_charm_rcl

  ! This subroutine sets the charm quark as "not light". This means 
  ! that, if the charm-quark is massive, it is considered as heavy 
  ! (i.e full mass dependence is kept). If the charm-quark is 
  ! massless this subroutine has no effect (i.e. charm-quark 
  ! collinear singularities are regularized dimensionally).

  if (processes_generated) then
    if (warnings.le.warning_limit) then
      warnings = warnings + 1
      call openOutput
      write(nx,*)
      write(nx,*) 'WARNING: Processes already generated.'
      write(nx,*) '         The call of unset_light_charm_rcl ', &
                           'has no effects.'
      write(nx,*)
      call toomanywarnings
    endif
  endif

  light_c = .false.

  end subroutine unset_light_charm_rcl

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  subroutine unset_light_strange_rcl

  ! This subroutine sets the strange quark as "not light". This means 
  ! that, if the strange-quark is massive, it is considered as heavy 
  ! (i.e full mass dependence is kept). If the strange-quark is 
  ! massless this subroutine has no effect (i.e. strange-quark 
  ! collinear singularities are regularized dimensionally).

  if (processes_generated) then
    if (warnings.le.warning_limit) then
      warnings = warnings + 1
      call openOutput
      write(nx,*)
      write(nx,*) 'WARNING: Processes already generated.'
      write(nx,*) '         The call of unset_light_strange_rcl ', &
                           'has no effects.'
      write(nx,*)
      call toomanywarnings
    endif
  endif

  light_s = .false.

  end subroutine unset_light_strange_rcl

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  subroutine unset_light_top_rcl

  ! This subroutine sets the top quark as "not light". This means 
  ! that, if the top-quark is massive, it is considered as heavy 
  ! (i.e full mass dependence is kept). If the top-quark is massless 
  ! this subroutine has no effect (i.e. top-quark collinear 
  ! singularities are regularized dimensionally).

  if (processes_generated) then
    if (warnings.le.warning_limit) then
      warnings = warnings + 1
      call openOutput
      write(nx,*)
      write(nx,*) 'WARNING: Processes already generated.'
      write(nx,*) '         The call of unset_light_top_rcl ', &
                           'has no effects.'
      write(nx,*)
      call toomanywarnings
    endif
  endif

  light_t = .false.

  end subroutine unset_light_top_rcl

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  subroutine unset_light_bottom_rcl

  ! This subroutine sets the bottom quark as "not light". This means 
  ! that, if the bottom-quark is massive, it is considered as heavy 
  ! (i.e full mass dependence is kept). If the bottom-quark is 
  ! massless this subroutine has no effect (i.e. bottom-quark 
  ! collinear singularities are regularized dimensionally).

  if (processes_generated) then
    if (warnings.le.warning_limit) then
      warnings = warnings + 1
      call openOutput
      write(nx,*)
      write(nx,*) 'WARNING: Processes already generated.'
      write(nx,*) '         The call of unset_light_bottom_rcl ', &
                           'has no effects.'
      write(nx,*)
      call toomanywarnings
    endif
  endif

  light_b = .false.

  end subroutine unset_light_bottom_rcl

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  subroutine use_dim_reg_soft_rcl

  ! This subroutine selects dimensional regularization for soft 
  ! singularities.

  if (processes_generated) then
    if (warnings.le.warning_limit) then
      warnings = warnings + 1
      call openOutput
      write(nx,*)
      write(nx,*) 'WARNING: Processes already generated.'
      write(nx,*) '         The call of use_dim_reg_soft_rcl ', &
                           'has no effects.'
      write(nx,*)
      call toomanywarnings
    endif
  endif

  reg_soft = 1

  end subroutine use_dim_reg_soft_rcl

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  subroutine use_mass_reg_soft_rcl (m)

  ! This subroutine selects mass regularization for soft singularities 
  ! The mass regulator is set to "m". 
  ! REMARK: Mass regularization for soft singularities is allowed 
  !         only in combination with mass regularization for 
  !         collinear singularities of light charged fermions. 
  !         Therefore no external charged massless fermions are 
  !         allowed (and the code stops).

  real(dp), intent(in) :: m

  if (processes_generated) then
    if (warnings.le.warning_limit) then
      warnings = warnings + 1
      call openOutput
      write(nx,*)
      write(nx,*) 'WARNING: Processes already generated.'
      write(nx,*) '         The call of use_mass_reg_soft_rcl ', &
                           'with argument'
      write(nx,*) '          m = ',m
      write(nx,*) '         has no effects.'
      write(nx,*)
      call toomanywarnings
    endif
  endif

  if (m.lt.0d0) then
    if (warnings.le.warning_limit) then
      warnings = warnings + 1
      call openOutput
      write(nx,*)
      write(nx,*) &
        'ERROR: use_mass_reg_soft_rcl called with negative mass'
      write(nx,*)
      call toomanywarnings
    endif
    call istop (ifail,1)
  endif

  reg_soft = 2

  lambda  = m

  end subroutine use_mass_reg_soft_rcl

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  subroutine set_mass_reg_soft_rcl (m)

  ! This subroutine sets the mass regulator for soft singularities 
  ! to "m". 

  real(dp), intent(in) :: m

  if (processes_generated.and.dynamic_settings.eq.0) then
    if (warnings.le.warning_limit) then
      warnings = warnings + 1
      call openOutput
      write(nx,*)
      write(nx,*) 'WARNING: Processes already generated.'
      write(nx,*) '         The call of use_mass_reg_soft_rcl ', &
                           'with argument'
      write(nx,*) '          m = ',m
      write(nx,*) '         has no effects.'
      write(nx,*)
      call toomanywarnings
    endif
  endif

  if (m.lt.0d0) then
    if (warnings.le.warning_limit) then
      warnings = warnings + 1
      call openOutput
      write(nx,*)
      write(nx,*) &
        'ERROR: use_mass_reg_soft_rcl called with negative mass'
      write(nx,*)
      call toomanywarnings
    endif
    call istop (ifail,1)
  endif

  lambda  = m

  if (processes_generated) changed_lambda = .true.

  end subroutine set_mass_reg_soft_rcl

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  subroutine set_delta_uv_rcl (d)

  ! This subroutine sets coefficient "DeltaUV" of the UV pole 
  ! (1/ep - EulerConstant + ln(4*pi)) in the MSbar scheme to "d".

  real(dp), intent(in) :: d

  if (processes_generated.and.dynamic_settings.eq.0) then
    if (warnings.le.warning_limit) then
      warnings = warnings + 1
      call openOutput
      write(nx,*)
      write(nx,*) 'WARNING: Processes already generated.'
      write(nx,*) '         The call of set_delta_uv_rcl ', &
                           'with argument'
      write(nx,*) '          d = ',d
      write(nx,*) '         has no effects.'
      write(nx,*)
      call toomanywarnings
    endif
  endif

  DeltaUV = d

  if (processes_generated) changed_DeltaUV = .true.

  end subroutine set_delta_uv_rcl

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  subroutine set_mu_uv_rcl (m)

  ! This subroutine sets the UV scale "muUV" to "m".

  real(dp), intent(in) :: m

  if (processes_generated.and.dynamic_settings.eq.0) then
    if (warnings.le.warning_limit) then
      warnings = warnings + 1
      call openOutput
      write(nx,*)
      write(nx,*) 'WARNING: Processes already generated.'
      write(nx,*) '         The call of set_mu_uv_rcl with argument'
      write(nx,*) '          m = ',m
      write(nx,*) '         has no effects.'
      write(nx,*)
      call toomanywarnings
    endif
  endif

  if (m.lt.0d0) then
    if (warnings.le.warning_limit) then
      warnings = warnings + 1
      call openOutput
      write(nx,*)
      write(nx,*) &
        'ERROR: set_mu_uv_rcl called with negative mass'
      write(nx,*)
      call toomanywarnings
    endif
    call istop (ifail,1)
  endif

  muUV = m

  if (processes_generated) changed_muUV = .true.

  end subroutine set_mu_uv_rcl

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  subroutine set_delta_ir_rcl (d,d2)

  ! This subroutine sets the coefficients "DeltaIR" of the IR 
  ! single pole and "Deltair2" of the IR double pole to "d" and to 
  ! "d2" respectively. 

  real(dp), intent(in) :: d,d2

  if (processes_generated.and.dynamic_settings.eq.0) then
    if (warnings.le.warning_limit) then
      warnings = warnings + 1
      call openOutput
      write(nx,*)
      write(nx,*) 'WARNING: Processes already generated.'
      write(nx,*) '         The call of set_delta_ir_rcl ', &
                           'with arguments'
      write(nx,*) '          d  = ',d
      write(nx,*) '          d2 = ',d2
      write(nx,*) '         has no effects.'
      write(nx,*)
      call toomanywarnings
    endif
  endif

  DeltaIR  = d
  DeltaIR2 = d2

  if (processes_generated) changed_DeltaIR = .true.

  end subroutine set_delta_ir_rcl

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  subroutine set_mu_ir_rcl (m)

  ! This subroutine sets the IR scale "muIR" to "m".

  real(dp), intent(in) :: m

  if (processes_generated.and.dynamic_settings.eq.0) then
    if (warnings.le.warning_limit) then
      warnings = warnings + 1
      call openOutput
      write(nx,*)
      write(nx,*) 'WARNING: Processes already generated.'
      write(nx,*) '         The call of set_mu_ir_rcl with argument'
      write(nx,*) '          m = ',m
      write(nx,*) '         has no effects.'
      write(nx,*)
      call toomanywarnings
    endif
  endif

  if (m.lt.0d0) then
    if (warnings.le.warning_limit) then
      warnings = warnings + 1
      call openOutput
      write(nx,*)
      write(nx,*) &
        'ERROR: set_mu_ir_rcl called with negative mass'
      write(nx,*)
      call toomanywarnings
    endif
    call istop (ifail,1)
  endif

  muIR = m

  if (processes_generated) changed_muIR = .true.

  end subroutine set_mu_ir_rcl

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  subroutine set_alphas_rcl (a,s,Nf)

  ! This subroutine sets the values of alpha_s ("als") to "a", the 
  ! renormalization scale "Qren" to "s" and the number of active 
  ! quark flavours "Nfren" to "Nf". 
  ! The choosen value of "a" should correspond to the physical value 
  ! of alpha_s at the scale "s" computed in the "Nf"-flavour scheme.
  ! Accepted values for "Nf" are -1,3,4,5,6
  ! Nf = -1      -> all quarks with mass less than "s" are active 
  !                 flavours
  ! Nf = 3,4,5,6 -> the 3,4,5,6 lightest quarks are the active 
  !                 flavours

  integer,  intent(in) :: Nf
  real(dp), intent(in) :: a,s

  integer :: i

  if (a.le.0d0.or.s.le.0d0) then
    if (warnings.le.warning_limit) then
      warnings = warnings + 1
      call openOutput
      write(nx,*)
      write(nx,*) &
        'ERROR: set_alphas_rcl called ', &
               'with wrong arguments'
      write(nx,*)
      call toomanywarnings
    endif
    call istop (ifail,1)
  endif

  select case (Nf)
  case (-1,3,4,5,6)
  case default
    if (warnings.le.warning_limit) then
      warnings = warnings + 1
      call openOutput
      write(nx,*)
      write(nx,*) 'ERROR: set_alphas_rcl called ', &
                          'with a wrong number of '
      write(nx,*) '       light flavours Nf (accepted values are Nf = -1,3,4,5,6)'
      write(nx,*)
      call toomanywarnings
    endif
    call istop (ifail,1)
  end select

  als   = a
  Qren  = s
  Nfren = Nf

  if (processes_generated) then
    ! set Nlq, the number of active flavours for alpha_s renormalization. 
    select case (Nfren)
    case (-1)
      ! Nlq is the number of quarks lighter than Qren
      Nlq = 0
      do i = 1,6
        if (mq2(i).lt.Qren**2) Nlq = Nlq + 1
      enddo
    case (3,4,5)
      if (mq2(Nfren+1).eq.0d0) then
        if (warnings.le.warning_limit) then
          warnings = warnings + 1
          call openOutput
          write(nx,*)
          write(nx,*) 'ERROR: Wrong number Nfren of light flavours ', &
                             'for alphas renormaliztion'
          write(nx,*) '       (Nfren can not be smaller than the ', &
                             'number of massless quarks)'
          write(nx,*)
          call toomanywarnings
        endif
        call istop (ifail,1)
      endif
      Nlq = Nfren
    case (6)
      Nlq = Nfren
    case default
      if (warnings.le.warning_limit) then
        warnings = warnings + 1
        call openOutput
        write(nx,*)
        write(nx,*) 'ERROR: Wrong value for the number Nfren of light ', &
                           'flavours for alphas renormaliztion'
        write(nx,*) '       (accepted values are Nfren = -1,3,4,5,6)'
        write(nx,*)
        call toomanywarnings
      endif
      call istop (ifail,1)
    end select
  endif

  end subroutine set_alphas_rcl

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  subroutine get_alphas_rcl (a)

  ! This subroutine extracts the actual value of alpha_s ("als"). 

  real(dp), intent(out) :: a

  a = als

  end subroutine get_alphas_rcl

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  subroutine get_renormalization_scale_rcl (mu)

  ! This subroutine extracts the actual value of the renormalization 
  ! scale for alpha_s ("Qren"). 

  real(dp), intent(out) :: mu

  mu = Qren

  end subroutine get_renormalization_scale_rcl

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  subroutine get_flavour_scheme_rcl (Nf)

  ! This subroutine extracts the actual value of the flavour-scheme
  ! for the renormalization of alpha_s ("Nf"). 

  integer, intent(out) :: Nf

  Nf = Nfren

  end subroutine get_flavour_scheme_rcl

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  subroutine set_alphas_masses_rcl (mc,mb,mt,gc,gb,gt)

  ! This subroutine allows to set the values of the squared quark 
  ! masses for the running of alpha_s; their values are not used in 
  ! the rest of the computation.
  ! The squared masses of the 3 heaviest quarks are set to 
  ! "mc^2*sqrt(1+gc^2/mc^2)", "mb^2*sqrt(1+gb^2/mb^2)" and 
  ! "mt^2*sqrt(1+gt^2/mt^2)" respectively. 
  ! "mc", "mb", "mt" must satisfy 0 <= mc <= mb <= mt. 
  ! The optional arguments "gc", "gb", "gt"  are ignored if the 
  ! on-shell renormalization scheme is used.

  real(dp), intent(in)           :: mc,mb,mt
  real(dp), intent(in), optional :: gc,gb,gt

  if (processes_generated) then
    if (warnings.le.warning_limit) then
      warnings = warnings + 1
      call openOutput
      write(nx,*)
      write(nx,*) 'WARNING: Processes already generated.'
      write(nx,*) '         The call of set_alphas_masses_rcl ', &
                           'with arguments'
      write(nx,*) '          mc = ',mc
      write(nx,*) '          mb = ',mb
      write(nx,*) '          mt = ',mt
      if (present(gc)) &
      write(nx,*) '          gc = ',gc
      if (present(gb)) &
      write(nx,*) '          gb = ',gb
      if (present(gt)) &
      write(nx,*) '          gt = ',gt
      write(nx,*) '         has no effects.'
      write(nx,*)
      call toomanywarnings
    endif
  endif

  if ((mc.lt.0d0) .or. (mb .lt. mc) .or. (mt .lt. mb)) then
    if (warnings.le.warning_limit) then
      warnings = warnings + 1
      call openOutput
      write(nx,*)
      write(nx,*) &
        'ERROR: set_alphas_masses_rcl called with wrong arguments.'
      write(nx,*) &
        '       Masses need to be positive and to satisfy 0 <= mc <= mb <= mt.'
      write(nx,*)
      call toomanywarnings
    endif
    call istop (ifail,1)
  endif

  if (complex_mass_scheme .eq. 0 .and. &
      (present(gc) .or. present(gb) .or. present(gt))) then
    if (warnings.le.warning_limit) then
      warnings = warnings + 1
      call openOutput
      write(nx,*)
      write(nx,*) &
        'WARNING: Widths in set_alphas_masses_rcl are ignored'
      write(nx,*) &
        '         when complex_mass_scheme is 0.'
      write(nx,*)
      call toomanywarnings
    endif
  end if

  mq(1:3) = 0d0

  if (present(gc)) then
    if (gc.lt.0d0) then
      if (warnings.le.warning_limit) then
        warnings = warnings + 1
        call openOutput
        write(nx,*)
        write(nx,*) &
          'ERROR: set_alphas_masses_rcl called with wrong arguments.'
        write(nx,*) &
          '       Widths can not be negative.'
        write(nx,*)
        call toomanywarnings
      endif
      call istop (ifail,1)
    endif
    mq(4) = sqrt(abs(mc**2 - cId0*mc*gc))
  else
    mq(4) = mc
  endif

  if (present(gb)) then
    if (gb.lt.0d0) then
      if (warnings.le.warning_limit) then
        warnings = warnings + 1
        call openOutput
        write(nx,*)
        write(nx,*) &
          'ERROR: set_alphas_masses_rcl called with wrong arguments.'
        write(nx,*) &
          '       Widths can not be negative.'
        write(nx,*)
        call toomanywarnings
      endif
      call istop (ifail,1)
    endif
    mq(5) = sqrt(abs(mb**2 - cId0*mb*gb))
  else
    mq(5)   = mb
  endif

  if (present(gt)) then
    if (gt.lt.0d0) then
      if (warnings.le.warning_limit) then
        warnings = warnings + 1
        call openOutput
        write(nx,*)
        write(nx,*) &
          'ERROR: set_alphas_masses_rcl called with wrong arguments.'
        write(nx,*) &
          '       Widths can not be negative.'
        write(nx,*)
        call toomanywarnings
      endif
      call istop (ifail,1)
    endif
    mq(6) = sqrt(abs(mt**2 - cId0*mt*gt))
  else
    mq(6)   = mt
  endif

  use_active_qmasses = .true.

  end subroutine set_alphas_masses_rcl

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  subroutine use_gfermi_scheme_rcl (g,a)

  ! This subroutine sets the EW renormalization scheme to the gfermi 
  ! scheme (alpha is determined from gfermi) and, if present, sets 
  ! the Fermi constant "gf" to "g" or alternatively sets, if present, 
  ! alpha_gf ("algf") to "a".

  real(dp), optional, intent(in) :: g,a

  if (processes_generated) then
    if (warnings.le.warning_limit) then
      warnings = warnings + 1
      call openOutput
      write(nx,*)
      write(nx,*) 'WARNING: Processes already generated.'
      write(nx,*) '         The call of use_gfermi_scheme_rcl ', &
                           'has no effects.'
      write(nx,*)
      call toomanywarnings
    endif
  endif

  if (present(g).and.present(a)) then
    if (warnings.le.warning_limit) then
      warnings = warnings + 1
      call openOutput
      write(nx,*)
      write(nx,*) &
        'ERROR: use_gfermi_scheme_rcl called with two arguments'
      write(nx,*)
      call toomanywarnings
    endif
    call istop (ifail,1)
  endif

  if (present(g)) then
    if (g.lt.0d0) then
      if (warnings.le.warning_limit) then
        warnings = warnings + 1
        call openOutput
        write(nx,*)
        write(nx,*) &
          'ERROR: use_gfermi_scheme_rcl called with negative argument "g"'
        write(nx,*)
        call toomanywarnings
      endif
      call istop (ifail,1)
    endif
  endif

  if (present(a)) then
    if (a.lt.0d0) then
      if (warnings.le.warning_limit) then
        warnings = warnings + 1
        call openOutput
        write(nx,*)
        write(nx,*) &
          'ERROR: use_gfermi_scheme_rcl called with negative argument "a"'
        write(nx,*)
        call toomanywarnings
      endif
      call istop (ifail,1)
    endif
  endif

  ew_reno_scheme = 1

  if (present(g)) then
    gf   = g
    algf = - 1d0
  elseif (present(a)) then
    algf = a
  endif

  end subroutine use_gfermi_scheme_rcl

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  subroutine use_alpha0_scheme_rcl (a)

  ! This subroutine sets the EW renormalization scheme to the alpha0 
  ! scheme (alpha is renormalized at p^2 = 0) and, if present, sets 
  ! alpha(0) ("al0") to "a".

  real(dp), optional, intent(in) :: a

  if (processes_generated) then
    if (warnings.le.warning_limit) then
      warnings = warnings + 1
      call openOutput
      write(nx,*)
      write(nx,*) 'WARNING: Processes already generated.'
      write(nx,*) '         The call of use_alpha0_scheme_rcl ', &
                           'has no effects.'
      write(nx,*)
      call toomanywarnings
    endif
  endif

  if (present(a)) then
    if (a.lt.0d0) then
      if (warnings.le.warning_limit) then
        warnings = warnings + 1
        call openOutput
        write(nx,*)
        write(nx,*) &
          'ERROR: use_alpha0_scheme_rcl called with negative argument'
        write(nx,*)
        call toomanywarnings
      endif
      call istop (ifail,1)
    endif
  endif

  ew_reno_scheme = 2
  if (present(a)) al0 = a

  end subroutine use_alpha0_scheme_rcl

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  subroutine use_alphaz_scheme_rcl (a)

  ! This subroutine sets the EW renormalization scheme to the alphaz 
  ! scheme (alpha is renormalized at p^2 = mz^2) and, if present, sets 
  ! alpha(M_Z) ("alZ") to "a".

  real(dp), optional, intent(in) :: a

  if (processes_generated) then
    if (warnings.le.warning_limit) then
      warnings = warnings + 1
      call openOutput
      write(nx,*)
      write(nx,*) 'WARNING: Processes already generated.'
      write(nx,*) '         The call of use_alphaz_scheme_rcl ', &
                           'has no effects.'
      write(nx,*)
      call toomanywarnings
    endif
  endif

  if (present(a)) then
    if (a.lt.0d0) then
      if (warnings.le.warning_limit) then
        warnings = warnings + 1
        call openOutput
        write(nx,*)
        write(nx,*) &
          'ERROR: use_alphaz_scheme_rcl called with negative argument'
        write(nx,*)
        call toomanywarnings
      endif
      call istop (ifail,1)
    endif
  endif

  ew_reno_scheme = 3
  if (present(a)) alZ = a

  end subroutine use_alphaz_scheme_rcl

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  subroutine get_alpha_rcl (a)

  ! This subroutine extract the actual value of alpha. 

  real(dp), intent(out) :: a

  real(dp) :: rew2,rez2

  if (ew_reno_scheme.eq.1) then
    if (.not.processes_generated) then
      if (gf.lt.0d0) then
        alpha = algf
      else
        rew2 = mass_w*mass_w
        rez2 = mass_z*mass_z
        alpha = sq2*gf/pi*rew2*( 1d0 - rew2/rez2 )
      endif
    endif
    a = alpha
  elseif (ew_reno_scheme.eq.2) then
    a = al0
  elseif (ew_reno_scheme.eq.3) then
    a = alZ
  endif

  end subroutine get_alpha_rcl

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  subroutine set_complex_mass_scheme_rcl

  ! This subroutine selects the complex mass scheme of hep-ph/0505042 
  ! for renormalization

  if (processes_generated) then
    if (warnings.le.warning_limit) then
      warnings = warnings + 1
      call openOutput
      write(nx,*)
      write(nx,*) 'WARNING: Processes already generated.'
      write(nx,*) '         The call of ', &
                           'set_complex_mass_scheme_rcl ', &
                           'has no effects.'
      write(nx,*)
      call toomanywarnings
    endif
  endif

  complex_mass_scheme = 1

  end subroutine set_complex_mass_scheme_rcl

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  subroutine set_on_shell_scheme_rcl

  ! This subroutine selects the on-shell scheme for renormalization. 
  ! Just the real part of the self-energies are taken for the 
  ! computation of the counterterms. The imaginary part of the masses 
  ! are kept just in the denominator of propagators. 

  if (processes_generated) then
    if (warnings.le.warning_limit) then
      warnings = warnings + 1
      call openOutput
      write(nx,*)
      write(nx,*) 'WARNING: Processes already generated.'
      write(nx,*) '         The call of set_on_shell_scheme_rcl ', &
                           'has no effects.'
      write(nx,*)
      call toomanywarnings
    endif
  endif

  complex_mass_scheme = 0

  end subroutine set_on_shell_scheme_rcl

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  subroutine set_resonant_particle_rcl (pa)

  ! This subroutine marks particle "pa" as resonant. 
  ! "pa" can be the follwing characters:
  ! Scalars:       'H', 'p0', 'p+', 'p-'
  ! Vector bosons: 'g', 'A', 'Z', 'W+', 'W-'
  ! leptons:       'nu_e', 'nu_e~', 'e-', 'e+', 
  !                'nu_mu', 'nu_mu~', 'mu-', 'mu+', 
  !                'nu_tau', 'nu_tau~', 'tau-', 'tau+'
  ! quarks:        'u', 'u~', 'd', 'd~', 
  !                'c', 'c~', 's', 's~',
  !                't', 't~', 'b', 'b~'

  character(len=*), intent(in) :: pa

  integer :: npa

  if (processes_generated) then
    if (warnings.le.warning_limit) then
      warnings = warnings + 1
      call openOutput
      write(nx,*)
      write(nx,*) 'WARNING: Processes already generated.'
      write(nx,*) '         The call of set_resonant_particle_rcl ', &
                           'with argument'
      write(nx,*) '          pa = ',pa
      write(nx,*) '         has no effects.'
      write(nx,*)
      call toomanywarnings
    endif
  endif

  npa = npar(pa)

  select case (npa)
  case (3,8,12,17)
    resPar(3)      = .true.
    resPar(8)      = .true.
    resPar(12)     = .true.
    resPar(17)     = .true.
  case (4,5,9,10,13,14,18,19)
    resPar(4:5)    = .true.
    resPar(9:10)   = .true.
    resPar(13:14)  = .true.
    resPar(18:19)  = .true.
  case (11)
    resPar(11)     = .true.
  case (23,35)
    resPar(23)     = .true.
    resPar(35)     = .true.
  case (24,36)
    resPar(24)     = .true.
    resPar(36)     = .true.
  case (25,37)
    resPar(25)     = .true.
    resPar(37)     = .true.
  case (26,38)
    resPar(26)     = .true.
    resPar(38)     = .true.
  case (27,39)
    resPar(27)     = .true.
    resPar(39)     = .true.
  case (28,40)
    resPar(28)     = .true.
    resPar(40)     = .true.
  case (29,41)
    resPar(29)     = .true.
    resPar(41)     = .true.
  case (30,42)
    resPar(30)     = .true.
    resPar(42)     = .true.
  case (31,43)
    resPar(31)     = .true.
    resPar(43)     = .true.
  case default ! never the case
    resPar(npa)    = .true.
  end select

  end subroutine set_resonant_particle_rcl

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  subroutine switchon_resonant_selfenergies_rcl

  ! This subroutine sets the value of "resSE" to .true.

  if (processes_generated) then
    if (warnings.le.warning_limit) then
      warnings = warnings + 1
      call openOutput
      write(nx,*)
      write(nx,*) 'WARNING: Processes already generated.'
      write(nx,*) '         The call of ',                        &
                           'switchon_resonant_selfenergies_rcl ', &
                           'has no effects.'
      write(nx,*)
      call toomanywarnings
    endif
  endif

  resSE = .true.

  end subroutine switchon_resonant_selfenergies_rcl

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  subroutine switchoff_resonant_selfenergies_rcl

  ! This subroutine sets the value of "resSE" to .false.

  if (processes_generated) then
    if (warnings.le.warning_limit) then
      warnings = warnings + 1
      call openOutput
      write(nx,*)
      write(nx,*) 'WARNING: Processes already generated.'
      write(nx,*) '         The call of ',                         &
                           'switchoff_resonant_selfenergies_rcl ', &
                           'has no effects.'
      write(nx,*)
      call toomanywarnings
    endif
  endif

  resSE = .false.

  end subroutine switchoff_resonant_selfenergies_rcl

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  subroutine set_dynamic_settings_rcl (n)

  ! This routine regulates the dynamic setting of parameters:
  ! n = 0 -> Just alpha_s can be set after generate_processes_rcl
  ! n = 1 -> Also lambda, DeltaUV, muUV, DeltaIR2, DeltaIR and muIR 
  !          can be set after generate_processes_rcl

  integer, intent(in) :: n

  if (processes_generated) then
    if (warnings.le.warning_limit) then
      warnings = warnings + 1
      call openOutput
      write(nx,*)
      write(nx,*) 'WARNING: Processes already generated.'
      write(nx,*) '         The call of set_dynamic_settings_rcl ', &
                           'with arguments'
      write(nx,*) '          n = ',n
      write(nx,*) '         has no effects.'
      write(nx,*)
      call toomanywarnings
    endif
  endif

  dynamic_settings = n

  end subroutine set_dynamic_settings_rcl

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  subroutine set_momenta_correction_rcl (mc)

  ! This routine regulates the correction of external momenta given 
  ! by the user, when on-shell conditions or momentum conservation are 
  ! not fulfilled.
  ! mc = .true.  -> recola tries to correct the momenta
  ! mc = .false. -> recola does not try to correct the momenta
  ! When the correction does not work or is turned off, all 
  ! computed amplitudes with bad momenta are set to zero.

  logical, intent(in) :: mc

  momenta_correction = mc

  end subroutine set_momenta_correction_rcl

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  subroutine set_draw_level_branches_rcl (n)

  ! This subroutine sets the options for the generation of a .tex 
  ! file, to draw the off-shell currents of the processes:
  ! n = 0 -> the .tex file is not generated.
  ! n = 1 -> the .tex file is generated without colour structures.
  ! n = 2 -> the .tex file is generated with colour structures.

  integer, intent(in) :: n

  if (processes_generated) then
    if (warnings.le.warning_limit) then
      warnings = warnings + 1
      call openOutput
      write(nx,*)
      write(nx,*) 'WARNING: Processes already generated.'
      write(nx,*) '         The call of ', &
                           'set_draw_level_branches_rcl ', &
                           'with argument'
      write(nx,*) '          n = ',n
      write(nx,*) '         has no effects.'
      write(nx,*)
      call toomanywarnings
    endif
  endif

  if (n.lt.0.or.n.gt.2) then
    if (warnings.le.warning_limit) then
      warnings = warnings + 1
      call openOutput
      write(nx,*)
      write(nx,*) &
        'ERROR: set_draw_level_branches_rcl called with wrong argument'
      write(nx,*)
      call toomanywarnings
    endif
    call istop (ifail,1)
  endif

  draw = n

  end subroutine set_draw_level_branches_rcl

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  subroutine set_print_level_amplitude_rcl (n)

  ! This subroutine sets the level for printing the amplitude to "n":
  ! n = 0 -> nothing is printed.
  ! n = 1 -> The non vanishining amplitudes A(c,h) are printed at LO 
  !          and NLO (if NLO is computed) for each coulour structure 
  !          "c" and helicity "h"
  ! n = 2 -> In addition to n=1, the NLO (if computed) contributions 
  !          to A(c,h) from bare-loop, counterterms and rational part 
  !          are also printed.

  integer, intent(in) :: n

  if (n.lt.0.or.n.gt.2) then
    if (warnings.le.warning_limit) then
      warnings = warnings + 1
      call openOutput
      write(nx,*)
      write(nx,*) &
        'ERROR: set_print_level_amplitude_rcl called with wrong argument'
      write(nx,*)
      call toomanywarnings
    endif
    call istop (ifail,1)
  endif

  writeMat = n

  end subroutine set_print_level_amplitude_rcl

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  subroutine set_print_level_squared_amplitude_rcl (n)

  ! This subroutine sets the level for printing the squared amplitude 
  ! to "n":
  ! n = 0 -> nothing is printed.
  ! n = 1 -> The non vanishining squared amplitudes A2 
  !          summed/averaged over colour and helicities are 
  !          printed at LO and NLO (if NLO is computed).
  ! n = 2 -> In addition to n=1, the NLO (if computed) contributions 
  !          to A2 from bare-loop, counterterms and rational part 
  !          are also printed.
  ! n = 3 -> In addition to n=2, also the squared amplitudes A2h 
  !          summed/averaged over colour are printed at LO and NLO 
  !          (if NLO is computed) for each helicity "h".

  integer, intent(in) :: n

  if (n.lt.0.or.n.gt.3) then
    if (warnings.le.warning_limit) then
      warnings = warnings + 1
      call openOutput
      write(nx,*)
      write(nx,*) &
        'ERROR: set_print_level_squared_amplitude_rcl called with wrong argument'
      write(nx,*)
      call toomanywarnings
    endif
    call istop (ifail,1)
  endif

  writeMat2 = n

  end subroutine set_print_level_squared_amplitude_rcl

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  subroutine set_print_level_correlations_rcl (n)

  ! This subroutine sets the level for printing colour correlation 
  ! (if computed) to "n":
  ! n = 0 -> nothing is printed.
  ! n = 1 -> The non vanishining colour-correlated Born squared 
  !          amplitudes A2c(n,m) are printed for each emitter "n" and 
  !          spectator "m".

  integer, intent(in) :: n

  if (n.lt.0.or.n.gt.1) then
    if (warnings.le.warning_limit) then
      warnings = warnings + 1
      call openOutput
      write(nx,*)
      write(nx,*) &
        'ERROR: set_print_level_correlations_rcl called with wrong argument'
      write(nx,*)
      call toomanywarnings
    endif
    call istop (ifail,1)
  endif

  writeCor = n

  end subroutine set_print_level_correlations_rcl

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  subroutine set_print_level_RAM_rcl (n)

  ! This subroutine sets to "n" the level for printing RAM needed in 
  ! the present run of Recola:
  ! n = 0 -> nothing is printed.
  ! n = 1 -> RAM permanently used by process generation and process 
  !          computation is printed.
  ! n = 2 -> In addition to n=1, RAM temporally used by process 
  !          generation is printed.

  integer, intent(in) :: n

  if (processes_generated) then
    if (warnings.le.warning_limit) then
      warnings = warnings + 1
      call openOutput
      write(nx,*)
      write(nx,*) 'WARNING: Processes already generated.'
      write(nx,*) '         The call of set_print_level_RAM_rcl ', &
                           'with argument'
      write(nx,*) '          n = ',n
      write(nx,*) '         has no effects.'
      write(nx,*)
      call toomanywarnings
    endif
  endif

  if (n.lt.0.or.n.gt.2) then
    if (warnings.le.warning_limit) then
      warnings = warnings + 1
      call openOutput
      write(nx,*)
      write(nx,*) &
        'ERROR: set_print_level_RAM_rcl called with wrong argument'
      write(nx,*)
      call toomanywarnings
    endif
    call istop (ifail,1)
  endif

  writeRAM = n

  end subroutine set_print_level_RAM_rcl

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  subroutine scale_coupling3_rcl (fac,pa1,pa2,pa3)

  ! This subroutine scales the 3-particle coupling between particles 
  ! "pa1,pa2,pa3" by a factor "fac".
  ! "pa1,pa2,pa3" can be the follwing characters:
  ! Scalars:       'H', 'p0', 'p+', 'p-'
  ! Vector bosons: 'g', 'A', 'Z', 'W+', 'W-'
  ! leptons:       'nu_e', 'nu_e~', 'e-', 'e+', 
  !                'nu_mu', 'nu_mu~', 'mu-', 'mu+', 
  !                'nu_tau', 'nu_tau~', 'tau-', 'tau+'
  ! quarks:        'u', 'u~', 'd', 'd~', 
  !                'c', 'c~', 's', 's~',
  !                't', 't~', 'b', 'b~'
  ! REMARK: the rescaling on the counterterms and rational terms is 
  !         not yet implemented

  complex(dp), intent(in) :: fac
  character(len=*), intent(in) :: pa1,pa2,pa3

  integer :: npa1,npa2,npa3

  if (processes_generated) then
    if (warnings.le.warning_limit) then
      warnings = warnings + 1
      call openOutput
      write(nx,*)
      write(nx,*) 'WARNING: Processes already generated.'
      write(nx,*) '         The call of scale_coupling3_rcl ', &
                           'with arguments'
      write(nx,*) '          fac = ',fac
      write(nx,*) '          pa1 = ',pa1
      write(nx,*) '          pa2 = ',pa2
      write(nx,*) '          pa3 = ',pa3
      write(nx,*) '         has no effects.'
      write(nx,*)
      call toomanywarnings
    endif
  endif

  npa1 = npar(pa1)
  npa2 = npar(pa2)
  npa3 = npar(pa3)

  coupling3(npa1,npa2,npa3) = fac
  coupling3(npa1,npa3,npa2) = fac
  coupling3(npa2,npa1,npa3) = fac
  coupling3(npa2,npa3,npa1) = fac
  coupling3(npa3,npa1,npa2) = fac
  coupling3(npa3,npa2,npa1) = fac

  end subroutine scale_coupling3_rcl

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  subroutine scale_coupling4_rcl (fac,pa1,pa2,pa3,pa4)

  ! This subroutine scales the 4-particle coupling between particles 
  ! "pa1,pa2,pa3,pa4" by a factor "fac".
  ! "pa1,pa2,pa3,pa4" can be the follwing characters:
  ! Scalars:       'H', 'p0', 'p+', 'p-'
  ! Vector bosons: 'g', 'A', 'Z', 'W+', 'W-'
  ! leptons:       'nu_e', 'nu_e~', 'e-', 'e+', 
  !                'nu_mu', 'nu_mu~', 'mu-', 'mu+', 
  !                'nu_tau', 'nu_tau~', 'tau-', 'tau+'
  ! quarks:        'u', 'u~', 'd', 'd~', 
  !                'c', 'c~', 's', 's~',
  !                't', 't~', 'b', 'b~'
  ! REMARK: the rescaling on the counterterms and rational terms is 
  !         not yet implemented

  complex(dp), intent(in) :: fac
  character(len=*), intent(in) :: pa1,pa2,pa3,pa4

  integer :: npa1,npa2,npa3,npa4

  if (processes_generated) then
    if (warnings.le.warning_limit) then
      warnings = warnings + 1
      call openOutput
      write(nx,*)
      write(nx,*) 'WARNING: Processes already generated.'
      write(nx,*) '         The call of scale_coupling4_rcl ', &
                           'with arguments'
      write(nx,*) '          fac = ',fac
      write(nx,*) '          pa1 = ',pa1
      write(nx,*) '          pa2 = ',pa2
      write(nx,*) '          pa3 = ',pa3
      write(nx,*) '          pa4 = ',pa4
      write(nx,*) '         has no effects.'
      write(nx,*)
      call toomanywarnings
    endif
  endif

  npa1 = npar(pa1)
  npa2 = npar(pa2)
  npa3 = npar(pa3)
  npa4 = npar(pa4)

  coupling4(npa1,npa2,npa3,npa4) = fac
  coupling4(npa1,npa2,npa4,npa3) = fac
  coupling4(npa1,npa3,npa2,npa4) = fac
  coupling4(npa1,npa3,npa4,npa2) = fac
  coupling4(npa1,npa4,npa2,npa3) = fac
  coupling4(npa1,npa4,npa3,npa2) = fac
  coupling4(npa2,npa1,npa3,npa4) = fac
  coupling4(npa2,npa1,npa4,npa3) = fac
  coupling4(npa2,npa3,npa1,npa4) = fac
  coupling4(npa2,npa3,npa4,npa1) = fac
  coupling4(npa2,npa4,npa1,npa3) = fac
  coupling4(npa2,npa4,npa3,npa1) = fac
  coupling4(npa3,npa1,npa2,npa4) = fac
  coupling4(npa3,npa1,npa4,npa2) = fac
  coupling4(npa3,npa2,npa1,npa4) = fac
  coupling4(npa3,npa2,npa4,npa1) = fac
  coupling4(npa3,npa4,npa1,npa2) = fac
  coupling4(npa3,npa4,npa2,npa1) = fac
  coupling4(npa4,npa1,npa2,npa3) = fac
  coupling4(npa4,npa1,npa3,npa2) = fac
  coupling4(npa4,npa2,npa1,npa3) = fac
  coupling4(npa4,npa2,npa3,npa1) = fac
  coupling4(npa4,npa3,npa1,npa2) = fac
  coupling4(npa4,npa3,npa2,npa1) = fac

  end subroutine scale_coupling4_rcl

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  subroutine switchoff_coupling3_rcl (pa1,pa2,pa3)

  ! This subroutine switches off the 3-particle coupling between 
  ! particles "pa1,pa2,pa3" .
  ! "pa1,pa2,pa3" can be the follwing characters:
  ! Scalars:       'H', 'p0', 'p+', 'p-'
  ! Vector bosons: 'g', 'A', 'Z', 'W+', 'W-'
  ! leptons:       'nu_e', 'nu_e~', 'e-', 'e+', 
  !                'nu_mu', 'nu_mu~', 'mu-', 'mu+', 
  !                'nu_tau', 'nu_tau~', 'tau-', 'tau+'
  ! quarks:        'u', 'u~', 'd', 'd~', 
  !                'c', 'c~', 's', 's~',
  !                't', 't~', 'b', 'b~'
  ! REMARK: the switching off on the counterterms and rational terms is 
  !         not yet implemented

  character(len=*), intent(in) :: pa1,pa2,pa3

  integer :: npa1,npa2,npa3

  if (processes_generated) then
    if (warnings.le.warning_limit) then
      warnings = warnings + 1
      call openOutput
      write(nx,*)
      write(nx,*) 'WARNING: Processes already generated.'
      write(nx,*) '         The call of switchoff_coupling3_rcl ', &
                           'with arguments'
      write(nx,*) '          pa1 = ',pa1
      write(nx,*) '          pa2 = ',pa2
      write(nx,*) '          pa3 = ',pa3
      write(nx,*) '         has no effects.'
      write(nx,*)
      call toomanywarnings
    endif
  endif

  npa1 = npar(pa1)
  npa2 = npar(pa2)
  npa3 = npar(pa3)

  coupling3(npa1,npa2,npa3) = 0d0
  coupling3(npa1,npa3,npa2) = 0d0
  coupling3(npa2,npa1,npa3) = 0d0
  coupling3(npa2,npa3,npa1) = 0d0
  coupling3(npa3,npa1,npa2) = 0d0
  coupling3(npa3,npa2,npa1) = 0d0

  end subroutine switchoff_coupling3_rcl

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  subroutine switchoff_coupling4_rcl (pa1,pa2,pa3,pa4)

  ! This subroutine switches off the 4-particle coupling between 
  ! particles "pa1,pa2,pa3,pa4".
  ! "pa1,pa2,pa3,pa4" can be the follwing characters:
  ! Scalars:       'H', 'p0', 'p+', 'p-'
  ! Vector bosons: 'g', 'A', 'Z', 'W+', 'W-'
  ! leptons:       'nu_e', 'nu_e~', 'e-', 'e+', 
  !                'nu_mu', 'nu_mu~', 'mu-', 'mu+', 
  !                'nu_tau', 'nu_tau~', 'tau-', 'tau+'
  ! quarks:        'u', 'u~', 'd', 'd~', 
  !                'c', 'c~', 's', 's~',
  !                't', 't~', 'b', 'b~'
  ! REMARK: the switching off on the counterterms and rational terms 
  !         is not yet implemented

  character(len=*), intent(in) :: pa1,pa2,pa3,pa4

  integer :: npa1,npa2,npa3,npa4

  if (processes_generated) then
    if (warnings.le.warning_limit) then
      warnings = warnings + 1
      call openOutput
      write(nx,*)
      write(nx,*) 'WARNING: Processes already generated.'
      write(nx,*) '         The call of switchoff_coupling4_rcl ', &
                           'with arguments'
      write(nx,*) '          pa1 = ',pa1
      write(nx,*) '          pa2 = ',pa2
      write(nx,*) '          pa3 = ',pa3
      write(nx,*) '          pa4 = ',pa4
      write(nx,*) '         has no effects.'
      write(nx,*)
      call toomanywarnings
    endif
  endif

  npa1 = npar(pa1)
  npa2 = npar(pa2)
  npa3 = npar(pa3)
  npa4 = npar(pa4)

  coupling4(npa1,npa2,npa3,npa4) = 0d0
  coupling4(npa1,npa2,npa4,npa3) = 0d0
  coupling4(npa1,npa3,npa2,npa4) = 0d0
  coupling4(npa1,npa3,npa4,npa2) = 0d0
  coupling4(npa1,npa4,npa2,npa3) = 0d0
  coupling4(npa1,npa4,npa3,npa2) = 0d0
  coupling4(npa2,npa1,npa3,npa4) = 0d0
  coupling4(npa2,npa1,npa4,npa3) = 0d0
  coupling4(npa2,npa3,npa1,npa4) = 0d0
  coupling4(npa2,npa3,npa4,npa1) = 0d0
  coupling4(npa2,npa4,npa1,npa3) = 0d0
  coupling4(npa2,npa4,npa3,npa1) = 0d0
  coupling4(npa3,npa1,npa2,npa4) = 0d0
  coupling4(npa3,npa1,npa4,npa2) = 0d0
  coupling4(npa3,npa2,npa1,npa4) = 0d0
  coupling4(npa3,npa2,npa4,npa1) = 0d0
  coupling4(npa3,npa4,npa1,npa2) = 0d0
  coupling4(npa3,npa4,npa2,npa1) = 0d0
  coupling4(npa4,npa1,npa2,npa3) = 0d0
  coupling4(npa4,npa1,npa3,npa2) = 0d0
  coupling4(npa4,npa2,npa1,npa3) = 0d0
  coupling4(npa4,npa2,npa3,npa1) = 0d0
  coupling4(npa4,npa3,npa1,npa2) = 0d0
  coupling4(npa4,npa3,npa2,npa1) = 0d0

  end subroutine switchoff_coupling4_rcl

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  subroutine set_ifail_rcl (i)

  ! This subroutine sets the ifail flag to "i". 
  ! The ifail flag regulates the behaviour of Recola in case of error.
  ! On input:
  ! ifail = -1 -> the code does not stop, when an error occurs
  ! ifail =  0 -> the code stops, when an error occurs

  integer, intent(in) :: i

  if (i.ne.0.and.i.ne.-1) then
    call openOutput
    write(nx,*)
    write(nx,*) &
      'ERROR: set_ifail_rcl called with wrong argument'
    write(nx,*)
    stop
  endif

  ifail = i

  end subroutine set_ifail_rcl

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  subroutine get_ifail_rcl (i)

  ! This subroutine extracts the actual value of the ifail flag in the
  ! output argument "i". 
  ! The ifail flag regulates the behaviour of Recola in case of error.
  ! On output:
  ! ifail = 0 -> no errors occured
  ! ifail = 1 -> an error occured because of a wrong call of 
  !              subroutines by the user
  ! ifail = 2 -> an error occured because of an internal bug of Recola

  integer, intent(out) :: i

  if (ifail.le.0) then; i = 0
  else;                 i = ifail
  endif

  end subroutine get_ifail_rcl

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  subroutine set_output_file_rcl (x)

  ! This subroutine sets the output file name to "x". 
  ! Any output produced by recola is written to the file named "x", 
  ! which is created with the first call of set_output_file_rcl with 
  ! argument "x" (if it already exists, its content is cleared). 
  ! Any output produced by recola is written to the file named "x", 
  ! which is created with the first call of set_output_file_rcl with 
  ! argument "x" (if it already exists, its content is cleared). 
  ! By a subsequent call of set_output_file_rcl with a different
  ! argument "y", the file "x" remains open (until the call
  ! reset_recola_rcl), but the output is now printed on "y".
  ! If the set_output_file_rcl is again called with argument
  ! "x", further output is then appended to the open file "x".
  ! By the call of reset_recola_rcl all opened output files are
  ! closed and, by further call of set_output_file_rcl with argument 
  ! "x", the content of the existing file "x" is cleared.

  character(*), intent(in) :: x

  if (len(trim(x)).gt.99) then
    if (warnings.le.warning_limit) then
      warnings = warnings + 1
      write(*,*)
      write(*,*) ' WARNING:'
      write(*,*) ' the name of output-file'
      write(*,*)
      write(*,'(a)') trim(x)
      write(*,*)
      write(*,*) ' is too long and has been truncated to'
      write(*,*)
      write(*,'(a99)') trim(x)
      write(*,*)
      if (warnings.eq.warning_limit+1) then
        write(*,*)
        write(*,*) ' Too many WARNINGs for this run:'
        write(*,*) ' Further WARNINGs will not be printed'
        write(*,*)
      endif
    endif
  endif

  outputfile = trim(x)

  if (recolascreen.eq.1) then
    write(*,*)
    write(*,'(1x,75("x"))')
    write(*,*)
    if (trim(outputfile).eq.'*') then
      write(*,*) 'From now on the output of RECOLA is written ', &
                 'on the screen'
    else
      write(*,*) 'From now on the output of RECOLA is written ', &
                 'to the file ',trim(outputfile)
    endif
    write(*,*)
    write(*,'(1x,75("x"))')
    write(*,*)
  endif

  end subroutine set_output_file_rcl

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  end module input_rcl

!#####################################################################
